/**
 * 用来发送XMLHTTP请求的帮助类
 * @constructor
 * @author yangmc
 * @version 1.0
 */
function XMLHttpHelper(){
 	var _sender = null;

 	/**
 	 * 同步请求
 	 */
 	this.sendRequestSync = function(url,xmlDoc){
 		if(_sender == null){
 			_sender = new asu.utils.xml.BRequest();
 		}
 		return _sender.sendRequest(url,xmlDoc).responseXML.getInnerDocument();
 	}

 	/**
 	 * 异步请求
 	 */
 	 this.sendRequestAsync = function(url,xmlDoc,callback){
 		if(_sender == null){
 			_sender = new asu.utils.xml.BRequest();
 		}
 		_sender.sendRequest(url,xmlDoc,callback,false);
 	}

 	/**
 	 * 取消请求
 	 */
 	this.abort = function(){
 		if(_sender == null){
 			return;
 		}

 		_sender.abort();
 	}
 }

/**
 * XMLHttpHelper的静态成员函数,负责发送请求
 */
 XMLHttpHelper.send = function(url,xmlDoc,callback){
 	if(XMLHttpHelper.instance == undefined){
 		XMLHttpHelper.instance = new XMLHttpHelper();
 	}
 	var xmlHttpHelper = XMLHttpHelper.instance;
 	if(callback == undefined || callback == null || typeof(callback) != 'function'){//同步
 		return xmlHttpHelper.sendRequestSync(url,xmlDoc);
 	}else{//异步
 		xmlHttpHelper.sendRequestAsync(url,xmlDoc,doCallback);
 	}

 	function doCallback(response){
 		callback(response.responseXML.getInnerDocument());
 	}
 }

 /**
  * XMLHttpHelper的静态成员函数,负责发送请求，并显示渲染后的结果放在div容器中
  */
 XMLHttpHelper.render = function(xml,xslUrl,div){
 	var xmlDoc = null;
 	if(typeof(xml) == 'object'){
 		xmlDoc = xml;
 	}else{
 		xmlDoc = XMLHttpHelper.send(xml,'');
 	}

 	var xslDoc = XMLHttpHelper.send(xslUrl,'');

 	div.innerHTML = xmlDoc.transformNode(xslDoc);
 }

/**
 * XMLHttpHelper的静态成员函数,支持带参数的XSL渲染
 * @param {Array} params:参数数组，数组的每一个对象，都应具有两个属性（paramName：传入xslProcessor的参数的名字，paramValue：传入xslProcessor的参数的值）
 */
 XMLHttpHelper.renderWithParam = function(xml,xslUrl,div,params){
 	var xmlDoc = null;
 	if(typeof(xml) == 'object'){
 		xmlDoc = xml;
 	}else{
 		xmlDoc = XMLHttpHelper.send(xml,'');
 	}

	var xslDoc = new ActiveXObject("MSXML2.FreeThreadedDOMDocument");
	xslDoc.async = false;//设置不异步调用
	xslDoc.load(xslUrl);

	var xslTemplate = new ActiveXObject("MSXML2.XSLTemplate");
	xslTemplate.stylesheet = xslDoc;

	var xslProcessor = xslTemplate.createProcessor();
	xslProcessor.input = xmlDoc;

	for(var i = 0; i < params.length; i++){
		xslProcessor.addParameter(params[i].paramName,params[i].paramValue);
	}

	xslProcessor.transform();
	div.innerHTML = xslProcessor.output;
 }

 /**
  * 取消发送
  */
 XMLHttpHelper.abort = function(){
 	if(XMLHttpHelper.instance == undefined){
 		return;
 	}
 	XMLHttpHelper.instance.abort();
 }


