Type.registerNamespace("asu.utils.xml");

/**
 * Xml Dom对象类工厂
 * @constructor 根据不同的客户端创建不同的XML DOM对象
 * @author:knight
 * @version 1.0
 */
asu.utils.xml.DocumentFactory = function(){

	this.createDocument = function(){
		return new ActiveXObject("Microsoft.XMLDOM");
	}
}

asu.utils.xml.DocumentFactory.registerClass('asu.utils.xml.DocumentFactory');

/**
 * BDocument调用方法中需要返回给用户的信息
 * @constructor
 */
asu.utils.xml.Message = function(){
	this.INVALID_DOM = "无效的XMLDocument对象";
	this.NULL_NODE_BY_XPATH = "根据指定的Xpath不能获取相应的节点";
	this.NULL_PARENT_NODE = "无法在NULL中插入XML子节点";
}

asu.utils.xml.Message.registerClass('asu.utils.xml.Message');

/**
 * 封装了常用的XML操作
 * @constructor:符合XML规范的字符串或者XMLDocument对象
 * @author:knight
 * @version:1.0
 * @copyright:bingosoft.net
 */
asu.utils.xml.BDocument = function(xml,async){
	var _document = null;
	var _async    = false;
	var _message  = new asu.utils.xml.Message();
	if (arguments.length>1)
		_async    = arguments[1];

	if ("string" == typeof(xml)){

		var domFactory = new asu.utils.xml.DocumentFactory();
		_document = domFactory.createDocument();
		_document.async = _async;
		_document.loadXML(xml);
		if ("" == _document.xml){
			_document.load(xml);
			if ("" == _document.xml)
				throw _message.INVALID_DOM;
		}
	}
	else if("object" == typeof(xml)){

		if ("" == xml.xml)
			throw _message.INVALID_DOM;
		else
			_document = xml;
	}
	else{
		throw _message.INVALID_DOM;
	}

	this.getInnerDocument = function(){
		return _document;
	}

	this.toString = function(){
		return _document.xml;
	}

	this.getRoot = function(){
		return _document.documentElement;
	}

	this.selectSingleNode = function(path){
		return _document.selectSingleNode(path);
	}

	/**
	 * 获取Xml Dom对象中符合给定的XPath的第一个节点的值
	 * @param {String} path寻找节点的Xpath表达式
	 * @param {Boolean} nullhandler是否处理无法寻找到节点的情况，当该参数设置
	 * 为true时，如果在指定path找不到节点，则抛出错误。该参数设置为false，或者调
	 * 用该函数时没有传入该参数，则在指定path找不到节点时，会返回空字符串代替
	 * @exception NULL_NODE_BY_XPATH(根据指定的Xpath不能获取相应的节点)
	 * @return {Object} 根据指定path所获取的第一个节点的值
	 */
	this.selectSingleNodeText = function(path,nullHandler){
		var node = _document.selectSingleNode(path);
 		if (node)
  			return node.text;
 		else{
 			if (arguments.length > 1 && true == arguments[1])
 				throw _message.NULL_NODE_BY_XPATH;
 			else
 				return "";
 		}
	}

	/**
	 * 向Xml Dom对象的根节点中插入子节点
	 * @param {String} nodeName 插入子节点的名称
	 * @param {String} tempText 插入子节点的值
	 * @return {XmlNode} 新插入的子节点
	 */
	this.appendNode = function(nodeName,tempText){
		 var nodeText = "";
		 if (arguments.length > 1)
		   	nodeText = arguments[1];

		 var node  = _document.createNode(1, nodeName, "");
		 node.text = nodeText;
		 this.getRoot().appendChild(node);

		 return node;
	}

	/**
	 * 向指定的节点中插入子节点
	 * @param {Object} parent 指定的父节点
	 * @param {String} nodeName 插入子节点的名称
	 * @param {String} tempText 插入子节点的值
	 * @exception NULL_PARENT_NODE（无法向Null中插入XML子节点）
	 */
	this.appendNodeAt = function(parent,nodeName,tempText){
		var parentNode = null;
		var nodeText   = "";
		if (arguments.length > 2)
		   nodeText = arguments[2];

		var node = _document.createNode(1, nodeName, "");
		node.text = nodeText;

		/**
		 * 根据parent获得指定的node，如果parent为String类型则认为
		 * parent为Xpath,通过Xpath来定位父节点
		 */
		if ("string" == typeof(parent))
			parentNode = _document.selectSingleNode(parent);
		else
			parentNode = parent;

		if (null == parentNode)
			throw _message.NULL_PARENT_NODE;

		parentNode.appendChild(node);

		return node;
	}

	/**
	 * 析构函数
	 * @author knight
	 */
    this.dispose = function() {
        _message  = null;
        _document = null;
    }
}

asu.utils.xml.BDocument.registerClass('asu.utils.xml.BDocument', null, Sys.IDisposable);