Type.registerNamespace("asu.utils.xml");

/**
 * 发送XMLHttpRequest请求的封装，使用示例：
 * function send(){
 *   var request = new asu.utils.xml.BRequest();
 *   '同步调用
 *   var result = request.sendRequest("Server.jsp",'').responseXML;
 *   '异步调用
 *   request.sendRequest("Server.jsp?id=1",'',callback);
 * }
 *
 * function callback(request){
 *	 alert(request.responseText);
 * }
 * @constructor
 * @author elvis
 * @version 1.0
 * @author knight
 * @version 2.0 2006-12-6
 */
asu.utils.xml.BRequest = function(){

    /*--  变量定义 --*/
	var _sendingRequests   = new Array();

	<!-- 属性定义 -->
	this.isAsync          = false;
	//请求的远端地址
	this.url              = null;
	//发送请求时的XML对象
	this.xmlDoc           = null;
	//内置的XMLHttpRequest对象
	this.innerRequest     = null;
	//是否自动显示错误窗口
	this.isAutoShowError  = true;
	//请求是否成功,默认为false
	this.success          = false;

	this.callback         = null;

	this.responseText     = null;

	this.responseXML      = null;

	var _errorTitle       = "";

	var _errorMessage     = "";

	/**
	 * abort
	 */
	this.abort = function(){
		for(var i=0;i<_sendingRequests.length;i++){
			_sendingRequests[i].innerRequest.abort();
		}
	}

    /**
	* @param {String} remoteURL
	* @param {Object} 发送XmlHttp请求的Xml对象或者Xml字符串
	* @param {Function} pacallBackFunction	异步调用的方法指针
	* @param {Boolean} 是否自动显示错误窗口
	* @return {BRequest} BRequest对象
    */
    this.sendRequest = function(remoteURL,xmlDoc,callBackFunction,autoShowError){

		var request 		 = new asu.utils.xml.BRequest();
		request.url 		 = remoteURL;
		request.xmlDoc 		 = xmlDoc;

		if (arguments.length > 2 && typeof(callBackFunction) == 'function'){
			request.isAsync   = true;
            request.callback = callBackFunction;
		}

		if (arguments.length > 3)
			request.isAutoShowError = autoShowError;

		//如果是异步请求则首先放入队列中
		if (request.isAsync){
			innerSendRequest(request);
		}
		else{
			//如果是同步请求则立即进行处理
			innerSendRequest(request);
			processHttpRequestResult(request);

			if(true == request.isAutoShowError){
	        	request.showErrorMessage();
	        }
		}

		return request;
	}

	/**
	 * 内部发送一个XmlHttp请求
	 */
	function innerSendRequest(request){

		request.innerRequest = new ActiveXObject("Microsoft.XMLHTTP");

		if (request.isAsync){
			request.innerRequest.onreadystatechange = internalCallBack;
			_sendingRequests.push(request);
		}

		request.innerRequest.open("POST", request.url, request.isAsync);
		request.innerRequest.setRequestHeader("Content-Type","text/xml");
		request.innerRequest.send(getXmlDocContent(request.xmlDoc));
	}

	 processHttpRequestResult = function(request){

		if(request.innerRequest.status == 200){
			try
			{
	        	request.responseXML = new asu.utils.xml.BDocument(request.innerRequest.responseText);
				if(request.getErrorMessage() == ""){
					request.success   = true;
				}
	        	else{
	        		request.success   = false;
	        	}
			}
			catch(e){
				var message = new asu.utils.xml.Message();
				if (e == message.INVALID_DOM){
					//如果是无效的XMLDom对象错误，表明请求返回的是Text信息
					request.success 	 = true;
					request.responseText = trim(request.innerRequest.responseText);
				}
				else{
					alert(e.description);
				}
			}
        }else if(request.innerRequest.status == 0){
        	request.success   = true;
        }else{
        	request.success   = false;
        	request.responseXML = createErrorXML(request.innerRequest.responseText, "");
        }
    }

	/**
	 * 私有函数，返回发送XmlHttpRequest时Post的Xml Body
	 * @param {String} xmlDoc
	 */
    getXmlDocContent = function(xmlDoc){
        var xmlContent = "";

        if(xmlDoc != null){
	        if (typeof(xmlDoc.xml) != "undefined"){
	            xmlContent = xmlDoc.xml;
	        } else {
	            xmlContent = xmlDoc;
	        }
        }
        return xmlContent;
    }

	/**
	 *
	 * @param {String} errorTitle 设置自定义的错误信息提示标题，用户可以通过设置该属性
	 * 进行错误标题的自定义，否则则根据XMLHttpRequest请求对象返回的XML属性中的error节点
	 * 自动获取错误标题信息
	 */
	this.setErrorTitle = function(errorTitle){
		_errorTitle = errorTitle;
	}

	this.getErrorTitle = function(){
        if("" == _errorTitle && null != this.responseXML){
            _errorTitle += this.responseXML.selectSingleNodeText("//error");
        }
        return _errorTitle;
	}


	this.setErrorMessage = function(errorMessage){
		_errorMessage = errorMessage;
	}

	this.getErrorMessage = function(){
        _errorMessage += this.responseXML.selectSingleNodeText("//message");
		return _errorMessage;
	}

	this.showErrorMessage = function(){
        if(!this.success){
            reportError(this.getErrorTitle() + "\n" + this.getErrorMessage());
        }
	}

    function internalCallBack(){
        for(var i=0;i<_sendingRequests.length;i++){
            var request = _sendingRequests[i];
            if(request.innerRequest.readyState == 4){
                _sendingRequests.splice(i,1);
                processHttpRequestResult(request);
	            if(request.responseXML){
	            	request.callback(request);
	            }
	            if(request.isAutoShowError){
	            	request.showErrorMessage();
	            }
            }
        }
    }

	createErrorXML = function(errValue, errStack)
	{
		var _dom = new asu.utils.xml.BDocument("<error/>");
		_dom.appendNode("message",errValue);
		_dom.appendNode("stack",errStack);
		_dom.appendNode("client","true");
	 	return _dom;
	}

	reportError = function(message){
		var sFeature = "dialogHeight:460px;dialogWidth:450px";
		var obj = new Object();
		obj.windowObj = window;
		obj.msgObj = message;
		window.showModalDialog("/ocrm/html/resources/error/reportError.jsp", obj, sFeature);
	}

	trim = function(str)
	{
	    return str.replace(/(^[\s]*)|([\s]*$)/g, "");
	}
}



asu.utils.xml.BRequest.registerClass('asu.utils.xml.BRequest');